"""..."""
from dataclasses import dataclass

from aligo.types import DataClass


@dataclass
class StarredFileRequest(DataClass):
    """..."""
    drive_id: str = None
    file_id: str = None
    starred: bool = True
    custom_index_key: str = None

    def __post_init__(self):
        if self.starred:
            self.custom_index_key = 'starred_yes'
        else:
            self.custom_index_key = ''
        super(StarredFileRequest, self).__post_init__()



"""分享相关"""
from dataclasses import asdict
from typing import Iterator

from aligo.core import *
from aligo.core.Config import *
from aligo.request import *
from aligo.response import *
from aligo.types import *


class Share(BaseAligo):
    """分享相关"""

    def share_file(self, body: CreateShareLinkRequest) -> CreateShareLinkResponse:
        """分享文件, 支持批量分享

        目前(2021年07月17日)官方只开放分享部分类型文件
        """
        response = self._post(ADRIVE_V2_SHARE_LINK_CREATE, body=body)
        return self._result(response, CreateShareLinkResponse)

    def update_share(self, body: UpdateShareLinkRequest) -> UpdateShareLinkResponse:
        """更新分享, 如更新 密码, 有效期 等"""
        response = self._post(V2_SHARE_LINK_UPDATE, body=body)
        return self._result(response, UpdateShareLinkResponse)

    def cancel_share(self, body: CancelShareLinkRequest) -> CancelShareLinkResponse:
        """取消分享"""
        response = self._post(ADRIVE_V2_SHARE_LINK_CANCEL, body=body)
        return self._result(response, CancelShareLinkResponse)

    def batch_cancel_share(self, body: BatchCancelShareRequest) -> Iterator[BatchSubResponse]:
        """批量取消分享"""
        yield from self.batch_request(BatchRequest(
                requests=[BatchSubRequest(
                    id=share_id,
                    url='/share_link/cancel',
                    body=CancelShareLinkRequest(
                        share_id=share_id
                    )
                ) for share_id in body.share_id_list]
        ), CancelShareLinkResponse)

    def get_share_list(self, body: GetShareLinkListRequest = None) -> Iterator[ShareLinkSchema]:
        """获取自己的分享链接

        :param body: GetShareLinkListRequest对象
        :return: ShareLinkSchema对象的迭代器
        """
        yield from self._list_file(ADRIVE_V2_SHARE_LINK_LIST, body, GetShareLinkListResponse)

    # 处理其他人的分享
    def get_share_info(self, body: GetShareInfoRequest) -> GetShareInfoResponse:
        """..."""
        response = self._post(ADRIVE_V2_SHARE_LINK_GET_SHARE_BY_ANONYMOUS, body=body)
        share_info = self._result(response, GetShareInfoResponse)
        return share_info

    def get_share_token(self, body: GetShareTokenRequest) -> GetShareTokenResponse:
        """..."""
        response = self._post(V2_SHARE_LINK_GET_SHARE_TOKEN, body=body)
        share_token = self._result(response, GetShareTokenResponse)
        return share_token

    def get_share_file_list(self, body: GetShareFileListRequest, x_share_token: str) -> Iterator[BaseShareFile]:
        """..."""
        response = self._auth.post(V2_FILE_LIST, body=asdict(body), headers={'x-share-token': x_share_token})
        file_list = self._result(response, GetShareFileListResponse)
        if isinstance(file_list, Null):
            yield file_list
            return
        yield from file_list.items
        if file_list.next_marker != '':
            body.marker = file_list.next_marker
            yield from self.get_share_file_list(body=body, x_share_token=x_share_token)

    def get_share_file(self, body: GetShareFileRequest, x_share_token: str) -> BaseShareFile:
        """..."""
        response = self._auth.post(V2_FILE_GET, body=asdict(body), headers={'x-share-token': x_share_token})
        share_file = self._result(response, BaseShareFile)
        return share_file

    def get_share_link_download_url(self, body: GetShareLinkDownloadUrlRequest,
                                    x_share_token: str) -> GetShareLinkDownloadUrlResponse:
        """..."""
        response = self._auth.post(V2_FILE_GET_SHARE_LINK_DOWNLOAD_URL, body=asdict(body),
                                   headers={'x-share-token': x_share_token})
        download_url = self._result(response, GetShareLinkDownloadUrlResponse)
        return download_url

    def share_file_saveto_drive(self, body: ShareFileSaveToDriveRequest,
                                x_share_token: str) -> ShareFileSaveToDriveResponse:
        """..."""
        if body.to_drive_id is None:
            body.to_drive_id = self.default_drive_id
        response = self._auth.post(V2_FILE_COPY, body=asdict(body), headers={'x-share-token': x_share_token})
        return self._result(response, ShareFileSaveToDriveResponse, [201, 202])

    def batch_share_file_saveto_drive(self, body: BatchShareFileSaveToDriveRequest,
                                      x_share_token: str) -> Iterator[BatchShareFileSaveToDriveResponse]:
        """..."""
        if body.to_drive_id is None:
            body.to_drive_id = self.default_drive_id

        for file_id_list in self._list_split(body.file_id_list, self.BATCH_COUNT):
            response = self._auth.post(ADRIVE_V2_BATCH, body={
                "requests": [
                    {
                        "body": asdict(ShareFileSaveToDriveRequest(
                            file_id=file_id,
                            share_id=body.share_id,
                            to_parent_file_id=body.to_parent_file_id,
                            to_drive_id=body.to_drive_id,
                            overwrite=body.overwrite,
                            auto_rename=body.auto_rename,
                        )),
                        "headers": {
                            'Content-Type': 'application/json',
                        },
                        "id": file_id,
                        "method": 'POST',
                        "url": '/file/copy'
                    } for file_id in file_id_list
                ],
                "resource": 'file'
            }, headers={'x-share-token': x_share_token})

            if response.status_code != 200:
                yield Null(response)
                return

            for batch in response.json()['responses']:
                i = BatchSubResponse(**batch)
                if i.body:
                    i.body = BatchShareFileSaveToDriveResponse(**i.body)
                yield i



"""todo 解析其他客户端地址"""

from typing import Union, overload

import yaml

from aligo.core import *
from aligo.types import *


class Client(BaseAligo):
    """..."""

    @overload
    def get_latest_win32_client(self) -> ClientInfo:
        """..."""

    @overload
    def get_latest_win32_client(self) -> Null:
        """..."""

    def get_latest_win32_client(self) -> Union[ClientInfo, Null]:
        """获取最新win32客户端"""
        response = self._auth.get(path='/manifest/dtron/aDrive/win32/ia32/latest.yml', host='https://im.dingtalk.com')
        if response.status_code == 200:
            return ClientInfo(**yaml.safe_load(response.text))
        return Null(response)



"""drive"""

from aligo.core import *
from aligo.request import *
from aligo.types import *


class Drive(Core):
    """..."""

    def get_drive(self, drive_id: str = None) -> BaseDrive:
        """获取 drive"""
        body = GetDriveRequest(drive_id=drive_id)
        return super(Drive, self).get_drive(body)



"""..."""
from dataclasses import dataclass, field

from .DataClass import DataClass


@dataclass(eq=False)
class Token(DataClass):
    """..."""
    user_name: str = None
    nick_name: str = None
    user_id: str = None
    default_drive_id: str = None
    default_sbox_drive_id: str = None
    role: str = field(default=None, repr=False)
    status: str = field(default=None, repr=False)
    access_token: str = field(default=None, repr=False)
    refresh_token: str = field(default=None, repr=False)
    expires_in: int = field(default=None, repr=False)
    token_type: str = field(default=None, repr=False)
    avatar: str = field(default=None, repr=False)
    expire_time: str = field(default=None, repr=False)
    state: str = field(default=None, repr=False)
    exist_link: list = field(default=None, repr=False)
    need_link: bool = field(default=None, repr=False)
    user_data: dict = field(default=None, repr=False)
    pin_setup: bool = field(default=None, repr=False)
    is_first_login: bool = field(default=None, repr=False)
    need_rp_verify: bool = field(default=None, repr=False)
    device_id: str = field(default=None, repr=False)



"""一些枚举类型"""
from typing import Optional, Literal

MediaTranscodeStatus = Optional[
    Literal['running', 'finished', 'failed']
]

GetFileListOrderBy = Optional[
    Literal['name', 'created_at', 'updated_at', 'size']
]

SearchFileOrderBy = Optional[
    Literal['name', 'created_at', 'updated_at', 'size']
]

GetFileListFields = Optional[
    Literal['*', 'thumbnail']
]

GetFileFields = GetFileListFields

BaseFileCategory = Optional[
    Literal['others', 'doc', 'image', 'audio', 'video']
]

OrderDirection = Optional[
    Literal['ASC', 'DESC']
]

BaseFileType = Optional[
    Literal['file', 'folder']
]

CheckNameMode = Optional[
    Literal['auto_rename', 'refuse']
]

BaseFileContentHashName = Optional[
    Literal['sha1']
]

BaseFileStatus = Optional[
    Literal['uploading', 'available']
]

GetRecycleBinListOrderBy = Optional[
    Literal['name']
]

SharePolicy = Optional[
    Literal['url', 'msg']
]

GetShareFileListOrderBy = Optional[
    Literal['name', 'updated_at']
]

SearchCategory = Optional[
    Literal['image', 'video', 'audio', 'app', 'doc', 'others']
]

GetShareLinkListOrderBy = Optional[
    Literal['share_name', 'created_at', 'description', 'updated_at']
]

GetStarredListFields = Optional[
    Literal['*', 'thumbnail']
]



"""..."""
from dataclasses import dataclass, field
from typing import List

from aligo.types import DataClass
from .ShareItemInfo import ShareItemInfo


@dataclass
class GetShareInfoResponse(DataClass):
    """..."""
    avatar: str = None
    creator_id: str = None
    creator_name: str = None
    creator_phone: str = None
    expiration: str = None
    file_count: int = None
    file_infos: List[ShareItemInfo] = field(default_factory=list)
    share_name: str = None
    updated_at: str = None
    vip: str = None



"""..."""
from dataclasses import dataclass
from typing import List

from .DataClass import DataClass


@dataclass
class ClientInfo(DataClass):
    """..."""

    @dataclass
    class _File(DataClass):
        """..."""
        url: str = None
        sha512: str = None
        size: int = None

    version: str = None
    files: List[_File] = None
    path: str = None
    sha512: str = None
    releaseDate: str = None
    stagingPercentage: int = None



"""..."""
from dataclasses import dataclass

from aligo.types import *


@dataclass
class GetDownloadUrlResponse(DataClass):
    """..."""
    expiration: str
    method: str
    size: int
    url: str
    cdn_url: str
    internal_url: str
    ratelimit: RateLimit = None



"""..."""

from dataclasses import dataclass, field

from .DataClass import DataClass


# 1. 只 REPR 能理解的数据
# 2. 尽可能的少
# 3. 不知道的不处理, 默认


@dataclass
class BaseUser(DataClass):
    """..."""
    user_name: str = None
    user_id: str = None
    default_drive_id: str = None
    description: str = field(default=None, repr=False)
    nick_name: str = field(default=None, repr=False)
    email: str = field(default=None, repr=False)
    phone: str = field(default=None, repr=False)
    role: str = field(default=None, repr=False)
    status: str = field(default=None, repr=False)
    created_at: int = field(default=None, repr=False)
    domain_id: str = field(default=None, repr=False)
    updated_at: int = field(default=None, repr=False)
    avatar: str = field(default=None, repr=False)
    user_data: dict = field(default=None, repr=False)
    deny_change_password_by_self: bool = field(default=False, repr=False)
    need_change_password_next_login: bool = field(default=False, repr=False)
    permission: str = field(default=None, repr=False)



"""GetUploadUrlRequest"""

from dataclasses import dataclass, field
from typing import List

from aligo.types import *


@dataclass
class GetUploadUrlRequest(DataClass):
    """GetUploadUrlRequest"""
    drive_id: str = None
    file_id: str = None
    upload_id: str = None
    content_md5: str = None
    part_info_list: List[UploadPartInfo] = field(default_factory=list, repr=False)



"""..."""
from typing import Iterator

from aligo.core import *
from aligo.core.Config import *
from aligo.request import *
from aligo.response import *
from aligo.types import *


class Recyclebin(BaseAligo):
    """删除文件太过危险, 只提供移动文件到回收站的功能"""

    def move_file_to_trash(self, body: MoveFileToTrashRequest) -> MoveFileToTrashResponse:
        """移动文件到回收站"""
        response = self._post(V2_RECYCLEBIN_TRASH, body=body)
        return self._result(response, MoveFileToTrashResponse, [202, 204])

    def batch_move_to_trash(self, body: BatchMoveToTrashRequest) -> Iterator[BatchSubResponse]:
        """..."""
        if body.drive_id is None:
            body.drive_id = self.default_drive_id

        yield from self.batch_request(BatchRequest(
                requests=[BatchSubRequest(
                    id=file_id,
                    url='/recyclebin/trash',
                    body=MoveFileToTrashRequest(
                        drive_id=body.drive_id, file_id=file_id
                    )
                ) for file_id in body.file_id_list]
        ), MoveFileToTrashResponse)

    def restore_file(self, body: RestoreFileRequest) -> RestoreFileResponse:
        """恢复文件"""
        response = self._post(V2_RECYCLEBIN_RESTORE, body=body)
        return self._result(response, RestoreFileResponse, 204)

    def batch_restore_files(self, body: BatchRestoreRequest) -> Iterator[BatchSubResponse]:
        """..."""
        if body.drive_id is None:
            body.drive_id = self.default_drive_id

        yield from self.batch_request(BatchRequest(
                requests=[BatchSubRequest(
                    id=file_id,
                    url='/recyclebin/restore',
                    body=RestoreFileRequest(
                        drive_id=body.drive_id, file_id=file_id
                    )
                ) for file_id in body.file_id_list]
        ), RestoreFileResponse)

    def get_recyclebin_list(self, body: GetRecycleBinListRequest) -> Iterator[BaseFile]:
        """获取回收站文件列表"""
        yield from self._list_file(V2_RECYCLEBIN_LIST, body, GetRecycleBinListResponse)



"""aligo"""
from .types import *
from .request import *
from .response import *
from .core import *
from .apis import *

__title__ = 'aligo'
__description__ = 'apis lib for aliyundrive.'
__url__ = 'https://github.com/foyoux/aligo'
__version__ = '1.1.10'
__author__ = 'foyou'
__author_email__ = 'yimi.0822@qq.com'
__license__ = 'Apache 2.0'
__copyright__ = f'Copyright 2021 {__author__}'
__ide__ = 'PyCharm - https://www.jetbrains.com/pycharm/'



"""基本分享文件"""
from dataclasses import dataclass, field
from typing import List

from .DataClass import DataClass
from .Enum import BaseFileCategory, BaseFileType
from .ImageMedia import ImageMedia
from .VideoMedia import VideoMedia
from .VideoPreview import VideoPreview


@dataclass
class BaseShareFile(DataClass):
    """..."""
    share_id: str = None
    name: str = None
    size: int = None
    creator: str = None
    description: str = None
    category: BaseFileCategory = None
    download_url: str = None
    file_extension: str = None
    file_id: str = None
    thumbnail: str = None
    type: BaseFileType = None
    updated_at: str = None
    created_at: str = None
    url: str = None
    parent_file_id: str = None
    selected: bool = None
    image_media_metadata: ImageMedia = None
    video_media_metadata: VideoMedia = None
    video_preview_metadata: VideoPreview = None
    # 2021年07月29日16时15分41秒
    mime_extension: str = None
    mime_type: str = None
    punish_flag: int = 0
    # 2021年08月31日17时56分14秒
    action_list: List[str] = field(default_factory=list, repr=False)



"""..."""
from dataclasses import dataclass, field
from typing import List

from aligo.types import BaseFile
from aligo.types import DataClass


@dataclass
class GetFileListResponse(DataClass):
    """..."""
    items: List[BaseFile] = field(default_factory=list)
    next_marker: str = ''



"""..."""
from dataclasses import dataclass
from typing import Dict

from aligo.types import DataClass
from aligo.types.DataClass import DataType


@dataclass
class BatchSubRequest(DataClass):
    """..."""
    body: DataType
    id: str
    url: str
    headers: Dict = None
    method: str = 'POST'

    def __post_init__(self):
        self.headers = {"Content-Type": "application/json"}
        super(BatchSubRequest, self).__post_init__()



"""认证模块"""

import base64
import json
import logging
import os
import tempfile
import time
from dataclasses import asdict
from pathlib import Path
from typing import Callable, overload, List, NoReturn, Dict
from urllib import parse

import coloredlogs
import qrcode
import qrcode_terminal
import requests

from aligo.core.Config import *
from aligo.types import *
from aligo.types.Enum import *

_aligo = Path.home().joinpath('.aligo')
_aligo.mkdir(parents=True, exist_ok=True)


def get_configurations() -> List[str]:
    """获取配置文件列表"""
    list_: List[str] = []
    file_: os.DirEntry
    for file_ in os.scandir(_aligo):
        list_.append(os.path.splitext(file_.name)[0])
    return list_


class Auth:
    """..."""

    def debug_log(self, response: requests.Response) -> NoReturn:
        """打印错误日志, 便于分析调试"""
        r = response.request
        self.log.warning(f'[method status_code] {r.method} {response.status_code}')
        self.log.warning(f'[url] {response.url}')
        self.log.warning(f'[headers] {r.headers}')
        self.log.warning(f'[request body] {r.body}')
        self.log.warning(f'[response body] {response.text[:1024]}')

    def error_log_exit(self, response: requests.Response) -> NoReturn:
        """打印错误日志并退出"""
        self.debug_log(response)
        exit(-1)

    @overload
    def __init__(
            self,
            name: str = 'aligo',
            show: Callable[[str], NoReturn] = None,
            level=logging.DEBUG,
            loglog: bool = False
    ):
        """扫描二维码登录"""

    @overload
    def __init__(
            self,
            name: str = 'aligo',
            refresh_token: str = None,
            level=logging.DEBUG,
            loglog: bool = False
    ):
        """refresh_token 登录"""

    def __init__(
            self, name: str = 'aligo',
            refresh_token: str = None,
            show: Callable[[str], NoReturn] = None,
            level: int = logging.DEBUG,
            loglog: bool = False
    ):
        """登录验证

        :param name: (可选, 默认: aligo) 配置文件名称, 便于使用不同配置文件进行身份验证
        :param refresh_token:
        :param show: (可选) 显示二维码的函数
        :param level: (可选) 控制控制台输出
        :param loglog: (可选) 控制文件输出
        """
        self._name = _aligo.joinpath(f'{name}.json')

        self.log = logging.getLogger(f'{__name__}:{name}')

        # if level <= logging.DEBUG:
        #     fmt = '%(asctime)s.%(msecs)03d %(levelname)5s %(message)s :%(filename)s %(lineno)s'
        # else:
        fmt = '%(asctime)s.%(msecs)03d %(levelname)5s %(message)s'

        coloredlogs.install(
            level=level,
            logger=self.log,
            milliseconds=True,
            datefmt='%X',
            fmt=fmt
        )

        if loglog:
            logfile = logging.FileHandler(filename=str(self._name)[:-5] + '.log', mode='w', encoding='utf-8')
            logfile.setLevel(logging.DEBUG)
            formatter = logging.Formatter('%(asctime)s  %(filename)s  %(funcName)s : %(levelname)s  %(message)s',
                                          datefmt='%F %X')
            logfile.setFormatter(formatter)
            self.log.addHandler(logfile)

        self.log.info(f'Config {self._name}')
        self.log.info(f'日志等级 {logging.getLevelName(level)}')

        #
        self.session = requests.session()
        self.session.params.update(UNI_PARAMS)  # type:ignore
        self.session.headers.update(UNI_HEADERS)

        self.session.get(AUTH_HOST + V2_OAUTH_AUTHORIZE, params={
            'login_type': 'custom',
            'response_type': 'code',
            'redirect_uri': 'https://www.aliyundrive.com/sign/callback',
            'client_id': CLIENT_ID,
            'state': r'{"origin":"file://"}',
            # 'state': '{"origin":"https://www.aliyundrive.com"}',
        }, stream=True).close()

        #
        SESSIONID = self.session.cookies.get('SESSIONID')
        self.log.debug(f'SESSIONID {SESSIONID}')

        #
        self.token: Optional[Token] = None
        if show is None:
            if os.name == 'nt':
                self.log.debug('Windows 操作系统')
                show = self._show_windows
            else:
                self.log.debug('类 Unix 操作系统')
                show = self._show_console
        self._show = show

        if self._name.exists():
            self.log.info(f'加载配置文件 {self._name}')
            self.token = Token(**json.load(self._name.open()))
        else:
            if refresh_token:
                self.log.debug('使用 refresh_token 方式登录')
                self._refesh_token(refresh_token)
                return
            self.log.info('使用 扫描二维码 方式登录')
            self._login()

        #
        self.session.headers.update({
            'Authorization': f'Bearer {self.token.access_token}'
        })

    def _save(self) -> NoReturn:
        """保存配置文件"""
        self.log.info(f'保存配置文件: {self._name}')
        json.dump(asdict(self.token), self._name.open('w'))

    def _login(self):
        """登录"""
        self.log.info('开始登录 ...')
        response = self._login_by_qrcode()

        if response.status_code != 200:
            self.log.error('登录失败 ~')
            self.error_log_exit(response)

        bizExt = response.json()['content']['data']['bizExt']
        bizExt = base64.b64decode(bizExt).decode('gb18030')
        accessToken = json.loads(bizExt)['pds_login_result']['accessToken']

        # 使用accessToken持久化身份认证
        response = self.session.post(
            AUTH_HOST + V2_OAUTH_TOKEN_LOGIN,
            json={
                'token': accessToken
            }
        )
        goto = response.json()['goto']
        code: str = parse.parse_qs(parse.urlparse(goto).query)['code'][0]  # type: ignore

        response = self.session.post(
            WEBSV_HOST + TOKEN_GET,
            json={
                'code': code
            }
        )

        if response.status_code != 200:
            self.log.error(f'登陆失败 ~')
            self.error_log_exit(response)

        self.token = Token(**response.json())

        #
        self.log.info('登录成功 ~')
        self.log.info(
            f'username: {self.token.user_name} nickname: {self.token.nick_name} user_id: {self.token.user_id}')

        # 保存
        self._save()

    def _login_by_qrcode(self) -> requests.Response:
        """二维码登录"""
        response = self.session.get(
            PASSPORT_HOST + NEWLOGIN_QRCODE_GENERATE_DO
        )
        data = response.json()['content']['data']
        self._show(data['codeContent'])
        while True:
            response = self.session.post(
                PASSPORT_HOST + NEWLOGIN_QRCODE_QUERY_DO,
                data=data
            )
            login_data = response.json()['content']['data']
            qrCodeStatus = login_data['qrCodeStatus']
            self.log.info('等待扫描二维码 ...')
            if qrCodeStatus == 'NEW':
                # self.log.info('等待扫描二维码 ...')
                pass
            elif qrCodeStatus == 'SCANED':
                self.log.info('已扫描, 等待确认 ...')
            elif qrCodeStatus == 'CONFIRMED':
                self.log.info(f'已确认 (你可以关闭二维码图像了.)')
                return response
            else:
                self.log.warning('未知错误: 可能二维码已经过期.')
                self.error_log_exit(response)
            time.sleep(2)

    def _refesh_token(self, refresh_token=None):
        """刷新 token"""
        if refresh_token is None:
            refresh_token = self.token.refresh_token
        self.log.info('刷新 token ...')
        response = self.session.post(
            WEBSV_HOST + TOKEN_REFRESH,
            json={'refresh_token': refresh_token}
        )
        if response.status_code == 200:
            self.token = Token(**response.json())
            self.session.headers.update({
                'Authorization': f'Bearer {self.token.access_token}'
            })
            self._save()
        else:
            self.log.error('刷新 token 失败 ~')
            self.debug_log(response)
            self._login()
            # error_log_exit(response)

    def request(self, method: str, url: str,
                params: Dict = None, headers: Dict = None, data=None,
                files: object = None, verify: bool = None, body: Dict = None) -> requests.Response:
        """统一请求方法"""
        # 删除值为None的键
        if body is not None:
            body = {k: v for k, v in body.items() if v is not None}

        if data is not None and isinstance(data, dict):
            data = {k: v for k, v in data.items() if v is not None}

        for i in range(3):
            response = self.session.request(method=method, url=url, params=params,
                                            data=data, headers=headers, files=files,
                                            verify=verify, json=body)
            status_code = response.status_code
            self.log.info(
                f'{response.request.method} {response.url} {status_code} {response.headers.get("Content-Length", 0)}'
            )
            if status_code == 401 or (
                    # aims search 手机端apis
                    status_code == 400 and response.text == 'AccessToken is invalid. AccessTokenExpired'
            ):
                self._refesh_token()
                continue

            return response

        self.log.info(f'重试3次仍旧失败~')
        self.error_log_exit(response)

    def get(self, path: str, host: str = API_HOST, params: dict = None, headers: dict = None,
            verify: bool = None) -> requests.Response:
        """..."""
        return self.request(method='GET', url=host + path, params=params,
                            headers=headers, verify=verify)

    def post(self, path: str, host: str = API_HOST, params: dict = None, headers: dict = None, data: dict = None,
             files=None, verify: bool = None, body: dict = None) -> requests.Response:
        """..."""
        return self.request(method='POST', url=host + path, params=params, data=data,
                            headers=headers, files=files, verify=verify, body=body)

    @staticmethod
    def _show_console(qr_link: str) -> NoReturn:
        """
        在控制台上显示二维码
        :param qr_link: 二维码链接
        :return: NoReturn
        """
        qrcode_terminal.draw(qr_link)

    @staticmethod
    def _show_windows(qr_link: str) -> NoReturn:
        """
        通过 *.png 的关联应用程序显示 qrcode
        :param qr_link: 二维码链接
        :return: NoReturn
        """
        qr_img = qrcode.make(qr_link)
        png = tempfile.mktemp('.png')
        qr_img.save(png)
        os.startfile(png)



"""分享相关"""
from typing import List

from aligo.core import *
from aligo.request import *
from aligo.response import *
from aligo.types import *
from aligo.types.Enum import *


class Share(Core):
    """..."""

    def share_file(self,
                   file_id_list: List[str],
                   share_name: str = None,
                   share_pwd: str = None,
                   expiration: str = None,
                   drive_id: str = None,
                   description: str = None) -> CreateShareLinkResponse:
        """..."""
        body = CreateShareLinkRequest(
            file_id_list=file_id_list,
            share_name=share_name,
            share_pwd=share_pwd,
            expiration=expiration,
            drive_id=drive_id,
            description=description,
        )
        return super(Share, self).share_file(body)

    def update_share(self,
                     share_id: str,
                     share_pwd: str = None,
                     expiration: str = None,
                     description: str = None,
                     share_name: str = None) -> UpdateShareLinkResponse:
        """更新分享, 如更新 密码, 有效期 等"""
        body = UpdateShareLinkRequest(
            share_id=share_id,
            share_pwd=share_pwd,
            expiration=expiration,
            description=description,
            share_name=share_name,
        )
        return super(Share, self).update_share(body)

    def cancel_share(self, share_id: str) -> CancelShareLinkResponse:
        """取消分享"""
        body = CancelShareLinkRequest(share_id=share_id)
        return super(Share, self).cancel_share(body)

    def batch_cancel_share(self, share_id_list: List[str]) -> List[BatchSubResponse]:
        """批量取消分享"""
        body = BatchCancelShareRequest(share_id_list=share_id_list)
        result = super(Share, self).batch_cancel_share(body)
        return [i for i in result]

    def get_share_list(self,
                       limit: int = 100,
                       order_by: GetShareLinkListOrderBy = 'created_at',
                       order_direction: OrderDirection = 'DESC',
                       include_canceled: bool = False) -> List[ShareLinkSchema]:
        """获取自己的分享链接"""
        body = GetShareLinkListRequest(
            limit=limit,
            order_by=order_by,
            order_direction=order_direction,
            include_canceled=include_canceled,
        )
        result = super(Share, self).get_share_list(body)
        return [i for i in result]

    # 处理其他人的分享
    def get_share_info(self, share_id: str) -> GetShareInfoResponse:
        """..."""
        body = GetShareInfoRequest(share_id=share_id)
        return super(Share, self).get_share_info(body)

    def get_share_token(self, share_id: str, share_pwd: str = '') -> GetShareTokenResponse:
        """..."""
        body = GetShareTokenRequest(share_id=share_id, share_pwd=share_pwd)
        return super(Share, self).get_share_token(body)

    def get_share_file_list(
            self,
            share_id: str,
            share_token: str,
            body: GetShareFileListRequest = None,
            **kwargs
    ) -> List[BaseShareFile]:
        """..."""
        if body is None:
            body = GetShareFileListRequest(share_id=share_id, **kwargs)
        result = super(Share, self).get_share_file_list(body, share_token)
        return [i for i in result]

    def get_share_file(
            self,
            share_id: str,
            file_id: str,
            share_token: str,
            body: GetShareFileRequest = None,
            **kwargs
    ) -> BaseShareFile:
        """..."""
        if body is None:
            body = GetShareFileRequest(share_id=share_id, file_id=file_id, **kwargs)
        return super(Share, self).get_share_file(body, share_token)

    def get_share_link_download_url(
            self,
            share_id: str,
            file_id: str,
            share_token: str,
            body: GetShareLinkDownloadUrlRequest = None,
            **kwargs
    ) -> GetShareLinkDownloadUrlResponse:
        """..."""
        if body is None:
            body = GetShareLinkDownloadUrlRequest(share_id=share_id, file_id=file_id, **kwargs)
        return super(Share, self).get_share_link_download_url(body, share_token)

    def share_file_saveto_drive(
            self,
            share_id: str,
            file_id: str,
            share_token: str,
            to_parent_file_id: str = 'root',
            to_drive_id: str = None,
            new_name: str = None,
            body: ShareFileSaveToDriveRequest = None,
            **kwargs
    ) -> ShareFileSaveToDriveResponse:
        """..."""
        if body is None:
            body = ShareFileSaveToDriveRequest(
                share_id=share_id,
                file_id=file_id,
                to_parent_file_id=to_parent_file_id,
                to_drive_id=to_drive_id,
                new_name=new_name,
                **kwargs
            )
        return super(Share, self).share_file_saveto_drive(body, share_token)

    def batch_share_file_saveto_drive(
            self,
            share_id: str,
            file_id_list: List[str],
            share_token: str,
            to_parent_file_id: str = 'root',
            to_drive_id: str = None,
            body: BatchShareFileSaveToDriveRequest = None,
            **kwargs
    ) -> List[BatchShareFileSaveToDriveResponse]:
        """..."""
        if body is None:
            body = BatchShareFileSaveToDriveRequest(
                share_id=share_id,
                file_id_list=file_id_list,
                to_parent_file_id=to_parent_file_id,
                to_drive_id=to_drive_id,
                **kwargs
            )
        result = super(Share, self).batch_share_file_saveto_drive(body, share_token)
        return [i for i in result]



"""..."""
from typing import List

from aligo.core import *
from aligo.request import *
from aligo.types import *
from aligo.types.Enum import *


class Search(Core):
    """..."""

    def search_file(self, name: str = None, category: SearchCategory = None, drive_id: str = None,
                    body: SearchFileRequest = None, **kwargs) -> List[BaseFile]:
        """搜索文件"""
        if body is None:
            query = f'name match "{name}"'
            if category is not None:
                query += f' and category = "{category}"'
            body = SearchFileRequest(query=query, **kwargs)
        result = super(Search, self).search_file(body)
        return [i for i in result]

    def search_aims(self, keyword: str = None, category: BaseFileCategory = 'image', drive_id: str = None,
                    body: AimSearchRequest = None, **kwargs) -> List[BaseFile]:
        """搜索目标/标签"""
        if body is None:
            body = AimSearchRequest(
                query=f"keywords ='{keyword}' and type = 'file' and category = '{category}'",
                drive_id=drive_id,
                **kwargs
            )
        result = super(Search, self).search_aims(body)
        return [i for i in result]
